// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ImpactAggregation;

interface

uses Esmf,
     Fund.CommonDimensions;

type
  
  IImpactAggregationState = public interface
    property eloss: IVariable2Dimensional<Timestep, Region, double> read;
    property sloss: IVariable2Dimensional<Timestep, Region, double> read;   
    property loss: IVariable2Dimensional<Timestep, Region, double> read;
    
    property income: IParameter2Dimensional<Timestep, Region, double> read;
    
    property water: IParameter2Dimensional<Timestep, Region, double> read;
    property forests: IParameter2Dimensional<Timestep, Region, double> read;
    property heating: IParameter2Dimensional<Timestep, Region, double> read;
    property cooling: IParameter2Dimensional<Timestep, Region, double> read;
    property agcost: IParameter2Dimensional<Timestep, Region, double> read;
    property drycost: IParameter2Dimensional<Timestep, Region, double> read;
    property protcost: IParameter2Dimensional<Timestep, Region, double> read;
    property entercost: IParameter2Dimensional<Timestep, Region, double> read;
    property hurrdam: IParameter2Dimensional<Timestep, Region, double> read;
    property species: IParameter2Dimensional<Timestep, Region, double> read;
    property deadcost: IParameter2Dimensional<Timestep, Region, double> read;
    property morbcost: IParameter2Dimensional<Timestep, Region, double> read;
    property wetcost: IParameter2Dimensional<Timestep, Region, double> read;
    property leavecost: IParameter2Dimensional<Timestep, Region, double> read;
    
    property switchoffwater: Boolean read;
    property switchoffforests: Boolean read;
    property switchoffheating: Boolean read;
    property switchoffcooling: Boolean read;
    property switchoffagcost: Boolean read;
    property switchoffdrycost: Boolean read;
    property switchoffprotcost: Boolean read;
    property switchoffentercost: Boolean read;
    property switchoffhurrdam: Boolean read;
    property switchoffspecies: Boolean read;
    property switchoffdeadcost: Boolean read;
    property switchoffmorbcost: Boolean read;
    property switchoffwetcost: Boolean read;
    property switchoffleavecost: Boolean read;
  end;
  
  ImpactAggregationComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IImpactAggregationState);                
  end;
  
implementation

method ImpactAggregationComponent.Run(clock: Clock; state: IImpactAggregationState); 
begin
  var s:=state;
  var t:=clock.Current;
  
  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin
    s.eloss[t,r]:=Math.Min(
      0.0
        - iif(s.switchoffwater,    0.0, s.water[t,r])
        - iif(s.switchoffforests,  0.0, s.forests[t,r])
        - iif(s.switchoffheating,  0.0, s.heating[t,r])
        - iif(s.switchoffcooling,  0.0, s.cooling[t,r])
        - iif(s.switchoffagcost,   0.0, s.agcost[t,r])
        + iif(s.switchoffdrycost,  0.0, s.drycost[t,r])
        + iif(s.switchoffprotcost, 0.0, s.protcost[t,r])
        + iif(s.switchoffentercost,0.0, s.entercost[t,r])
        + iif(s.switchoffhurrdam,  0.0, s.hurrdam[t,r]),
      s.income[t,r]);
  end;
    
  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin
    s.sloss[t,r]:=0.0
      + iif(s.switchoffspecies,   0.0, s.species[t,r])
      + iif(s.switchoffdeadcost,  0.0, s.deadcost[t,r] )
      + iif(s.switchoffmorbcost,  0.0, s.morbcost[t,r])
      + iif(s.switchoffwetcost,   0.0, s.wetcost[t,r] )
      + iif(s.switchoffleavecost, 0.0, s.leavecost[t,r]);
  end;
  
  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin
    s.loss[t,r]:=(s.eloss[t,r] + s.sloss[t,r])*1000000000.0;
  end;  
  
end;

end.