// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.Welfare;

interface

uses System.Collections.Generic,
  Esmf,
  Fund.CommonDimensions;

type
    
  GlobalWelfareComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IGlobalWelfareState);    
  end;
  
  RegionalWelfareComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IRegionalWelfareState);    
  end;
  
  
implementation

method GlobalWelfareComponent.Run(clock: Clock; state: IGlobalWelfareState); 
begin
  var s:=state;
  var t:=clock.Current;
  
  if t>=Timestep.FromSimulationYear(50) then
  begin    
    var gamma:=s.consumption[Timestep.FromSimulationYear(50)]*100.0/s.population[Timestep.FromSimulationYear(50)];
    
    var lowerBound:=Math.Pow(1.0+(s.elasticityofmarginalutility-1.0)*gamma,-1.0/(s.elasticityofmarginalutility-1.0));
    
    var perCapitaConsumption:=Math.Max(
      s.consumption[t]/s.population[t],
      lowerBound);
          
    var U:=method(consumption: double): Double;
      begin
        if s.elasticityofmarginalutility=1.0 then
          Result:=s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Log(consumption)
        else
          Result:=s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Pow(consumption, 1.0-s.elasticityofmarginalutility)/(1.0-s.elasticityofmarginalutility);
      end;
      
    var DF:=method(year: Timestep): Double;
      begin
        Result:=Math.Pow(1.0+s.prtp, -(double(t.Value)-50.0));
      end;
      
    s.welfare[t]:=s.welfare[t-1] + ( U(perCapitaConsumption) * s.population[t] *  DF(t) );
    s.marginalwelfare[t]:=DF(t) * s.utilitycalibrationmultiplicative / Math.Pow(perCapitaConsumption, s.elasticityofmarginalutility);
  end
  else
    s.welfare[t]:=0;
  
end;

method RegionalWelfareComponent.Run(clock: Clock; state: IRegionalWelfareState); 
begin
  var s:=state;
  var t:=clock.Current;
  
  if t>=s.starttimestep then
  begin   
    var w: Double := s.cummulativewelfare[t-1];
        
    var U:=method(consumption: double): Double;
      begin
        if s.elasticityofmarginalutility=1.0 then
          Result:=s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Log(consumption)
        else
          Result:=s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Pow(consumption, 1.0-s.elasticityofmarginalutility)/(1.0-s.elasticityofmarginalutility);
      end;
      
    var DF:=method(year: Timestep): Double;
      begin
        Result:=Math.Pow(1.0+s.prtp, -(year.Value-s.starttimestep.Value));
      end;
      
    for each r: Region in Enum.GetValues(typeof(Region)) do
    begin      
//      var gamma:=s.consumption[Timestep.FromSimulationYear(50),r]*100.0/s.population[Timestep.FromSimulationYear(50),r];
//    
//      var lowerBound:=Math.Pow(1.0+(s.elasticityofmarginalutility-1.0)*gamma,-1.0/(s.elasticityofmarginalutility-1.0));
      var lowerBound:=1.0;
              
      var perCapitaConsumption:=Math.Max(
        s.consumption[t,r]/s.population[t,r],
        lowerBound);
      
      w:=w + ( s.welfareweight[r] * U(perCapitaConsumption) * s.population[t,r] *  DF(t) );
      s.marginalwelfare[t,r]:=DF(t) * s.welfareweight[r] * s.utilitycalibrationmultiplicative / Math.Pow(perCapitaConsumption, s.elasticityofmarginalutility);
    end;
    s.cummulativewelfare[t]:=w;
    
    if clock.IsLastTimestep then
    begin
      var welfaretoinfinity:=0.0;
                 
      for each r: Region in Enum.GetValues(typeof(Region)) do
      begin   
        var welfaretoinfinityinregion:=0.0;
        
        var income:=s.income[t,r]*1000000000.0;
        var incomegrowth:=s.income[t,r]/s.income[t-1,r];
        
        var lossInPercentOfConsumption:=s.loss[t,r]/(income*0.8);
        var lossgrowth:=s.loss[t,r]/s.loss[t-1,r];
        lossgrowth:=0.9998;
        //lossgrowth:=1.0;
        
        var population:=s.population[t,r];
                
        var clock2:=new Clock(Timestep.FromSimulationYear(t.Value+1), Timestep.FromSimulationYear(clock.Current.Value + 10000));        
        
        while not clock2.IsDone do
        begin
          var perCapitaConsumption:=(income*0.8)*(1.0-lossInPercentOfConsumption)/population;
          
          welfaretoinfinityinregion:=welfaretoinfinityinregion+( s.welfareweight[r] * U(perCapitaConsumption) * population *  DF(clock2.Current) );

          income:=income*incomegrowth;
          lossInPercentOfConsumption:=lossInPercentOfConsumption*lossgrowth;
                    
          clock2.Advance;
        end;                

        welfaretoinfinity:=welfaretoinfinity + welfaretoinfinityinregion;
      end;
      s.totalwelfare:=s.cummulativewelfare[t] + welfaretoinfinity;
    end;
  end
  else
    s.cummulativewelfare[t]:=0;
  end;

end.