// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses System.Collections;

type
  // Foreward class declarations
  Damages = class;
  Incomes = class;
  Populations = class;

  // Class that holds all output from one model run
  ModelOutput = public class
  private
    m_damages: Damages;
    m_incomes: Incomes;
    m_populations: Populations;
  public
    constructor;

    property Damages: Damages read m_damages write m_damages;
    property Incomes: Incomes read m_incomes write m_incomes;
    property Populations: Populations read m_populations write m_populations;
  end;

  // The damage in one year, region and sector in US$
  Damage = public class
  private
    m_year: Integer;
    m_region: integer;
    m_sector: Sector;
    m_damage: double;
  public
    constructor; 
    constructor(YearData: Integer; RegionData: integer; SectorData: Sector; DamageData: double); 

    property Year: Integer read m_year write m_year;
    property Region: integer read m_region write m_region;
    property Sector: Sector read m_sector write m_sector;
    property Damage: double read m_damage write m_damage;
  end;

  // Collection of all Damages for one model run
  Damages = public class
  private
    // TODO -oDavid Anthoff: Replace with less memory thing
    m_list: ArrayList;
    m_damages: array [0..LegacyConstants.NYear-1, 0..LegacyConstants.NoReg-1, 0..LegacyConstants.NoSector-1] of Damage;

    function GetCount: Integer;
  protected
    function GetItem(Index: Integer): Damage;
  public
    constructor;

    procedure Add(Damage: Damage);
    class function CalculateMarginalDamage(Damages1, Damages2: Damages): Damages; 

    function GetByYearRegionSector(Year: Integer; Region: integer; Sector: Sector): Damage;

    property Count: Integer read GetCount;
    property Item[Index: Integer]: Damage read GetItem;
  end;

  // GDP for each year and region in US$
  Incomes = public class
  private
    m_incomes: array [0..LegacyConstants.NYear-1, 0..LegacyConstants.NoReg-1] of double;
  protected
    function GetItem(Year: Integer; Region: integer): double;
  public
    constructor;

    procedure Add(Year: Integer; Region: integer; Income: double);
    property Item[Year: Integer; Region: integer]: double read GetItem;
  end;

  // Population for each year and region in absolute numbers
  Populations = public class
  private
    m_populations: array [0..LegacyConstants.NYear-1, 0..LegacyConstants.NoReg-1] of double;
  protected
    function GetItem(Year: Integer; Region: integer): double;
  public
    constructor;

    procedure Add(Year: Integer; Region: integer; Population: double);
    property Item[Year: Integer; Region: integer]: double read GetItem;
  end;

implementation

constructor ModelOutput;
begin
  inherited constructor();

  m_damages:=new Fund.Damages;
  m_incomes:=new Fund.Incomes;
  m_populations:=new Fund.Populations;
end;

constructor Damage(YearData: Integer; RegionData: integer; SectorData: Sector; DamageData: double);
begin
  inherited constructor();

  m_year:=YearData;
  m_region:=RegionData;
  m_sector:=SectorData;
  m_damage:=DamageData;
end;

constructor Damage;
begin
  inherited constructor();
end;

procedure Damages.Add(Damage: Damage);
begin
  m_list.Add(Damage);
  m_damages[Damage.Year-1, Damage.Region - 1, Ord(Damage.Sector)]:=Damage;
end;

class function Damages.CalculateMarginalDamage(Damages1,
  Damages2: Damages): Damages;
var
  i: Integer;
  i_marginalDamage: Damage;
  i_marginalDamages: Damages;
  i_damage1, i_damage2: Damage;
begin
  i_marginalDamages:=new Damages;

  for i:=0 to Damages1.Count - 1 do
  begin
    i_damage1:=Damages1.Item[i];
    i_damage2:=Damages2.GetByYearRegionSector(i_damage1.Year, i_damage1.Region, i_damage1.Sector);

    i_marginalDamage:=new Damage;
    i_marginalDamage.Year:=i_damage1.Year;
    i_marginalDamage.Region:=i_damage1.Region;
    i_marginalDamage.Sector:=i_damage1.Sector;
    i_marginalDamage.Damage:=(i_damage2.Damage - i_damage1.Damage)/10000000;
    i_marginalDamages.Add(i_marginalDamage);
  end;
  Result:=i_marginalDamages;
end;

constructor Damages;
begin
  inherited constructor;

  m_list:=new ArrayList(LegacyConstants.NYear*LegacyConstants.NoReg*LegacyConstants.NoSector);
end;

function Damages.GetByYearRegionSector(Year: Integer; Region: integer;
  Sector: Sector): Damage;
begin
  Result:=m_damages[Year-1, Region - 1, Ord(Sector)]
end;

function Damages.GetCount: Integer;
begin
  Result:=m_list.Count;
end;

function Damages.GetItem(Index: Integer): Damage;
begin
  Result:=Damage(m_list.Item[Index]);
end;

procedure Incomes.Add(Year: Integer; Region: integer; Income: double);
begin
  m_incomes[Year-1, Region - 1]:=Income;
end;

constructor Incomes;
begin
  inherited constructor;
end;

function Incomes.GetItem(Year: Integer; Region: integer): double;
begin
  Result:=m_incomes[Year-1, Region - 1];
end;

procedure Populations.Add(Year: Integer; Region: integer;
  Population: double);
begin
  m_populations[Year-1, Region-1]:=Population;
end;

constructor Populations;
begin
  inherited constructor();

end;

function Populations.GetItem(Year: Integer; Region: integer): double;
begin
  Result:=m_populations[Year-1, Region-1];
end;

end.
