// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses System.Collections.Generic,
     System.Xml,
     System.IO;

type
  
  RandomNumberGenerator = public (DotNet, MersenneTwister);
  
  SimulationManager = public class
  private
    _randomize: boolean;
    _rng: RandomNumberGenerator;
    _outputDisaggregatedData: boolean;
    _fileName: string;
    _runs: List<Run>:=new List<Run>;
    _scenarios: List<Scenario>:=new List<Scenario>;
    _outputVerbal: boolean;
    _outputInputParameters: boolean;
  protected
  public
    constructor(fileName: string);
    
    method Load;
    
    property Randomize: Boolean read _randomize;
    property Rng: RandomNumberGenerator read _rng;
    property OutputDisaggregatedData: boolean read _OutputDisaggregatedData;
    property Runs: List<Run> read _runs;
    property Scenarios: List<Scenario> read _scenarios;
    property OutputVerbal: Boolean read _outputVerbal;
    property OutputInputParameters: boolean read _outputInputParameters;
  end;
  
  RunMode = public (MarginalRun, 
                    TotalRun,
                    CEARun,
                    FullMarginalRun,
                    BGERun,
                    EpaMarginalRun,
                    MarginalRun2,
                    OptimalTaxation);
                    
  SWFtoOptimize = public (GlobalSwf, RegionalSwf);
  
  Run = public class
  private
    _mode: RunMode;
    _monteCarloRuns: Integer;
    _scenario: Scenario;
    _scengmt: boolean;
    _yearsToAggregate: Integer;
    _weightingCombination: String;
    _outputVerbal: Boolean;
  public
    property Scenario: Scenario read _scenario write _scenario;
    property MonteCarloRuns: Integer read _monteCarloRuns write _monteCarloRuns;
    property Mode: RunMode read _mode write _mode;
    property Scengmt: boolean read _scengmt write _scengmt;
    property YearsToAggregate: integer read _yearsToAggregate write _yearsToAggregate;
    property WeightingCombination: string read _weightingCombination write _weightingCombination;
    property OutputVerbal: boolean read _outputVerbal write _outputVerbal;
    property SWFToOptimise: nullable SWFtoOptimize read write;
    property CalculateMeanForMonteCarlo: boolean read write;
    property OutputAllMonteCarloRuns: Boolean read write;
  end;
  
  Scenario = public class
  private
    _id: integer;
    _name: string;
    _excelFiles: List<string>:=new List<string>;
  public
    property Id: integer read _id write _id;
    property Name: string read _name write _name;
    property ExcelFiles: List<string> read _excelFiles;
  end;
  
implementation

constructor SimulationManager(fileName: string); 
begin
  _fileName:=fileName;
end;

method SimulationManager.Load; 
begin
  var dom:=new System.Xml.XmlDocument;
  
  dom.Load(_fileName);
  
  var root:=dom.DocumentElement;
  
  _randomize:=Convert.ToBoolean(root.SelectSingleNode('/Simulation/GlobalParameters/Randomize').InnerText);
  
  if root.SelectSingleNode('/Simulation/GlobalParameters/RandomNumberGenerator')<>nil then
  begin
    var rngAsString:=root.SelectSingleNode('/Simulation/GlobalParameters/RandomNumberGenerator').InnerText.ToLowerInvariant;
    
    case rngAsString of
      'dotnet': _rng:=RandomNumberGenerator.DotNet;
      'mersennetwister': _rng:=RandomNumberGenerator.MersenneTwister;
      else raise new InvalidOperationException('The configuration value for the RandomNumberGenerator is not supported');
    end;
     
  end
  else
    _rng:=RandomNumberGenerator.DotNet;
    
  _outputDisaggregatedData:=Convert.ToBoolean(root.SelectSingleNode('/Simulation/GlobalParameters/OutputDisaggregatedData').InnerText);
  var defaultScenGmt:=Convert.ToBoolean(root.SelectSingleNode('/Simulation/GlobalParameters/Scengmt').InnerText);
  var defaultYearsToAggregate:=Convert.ToInt32(root.SelectSingleNode('/Simulation/GlobalParameters/YearsToAggregate').InnerText);
  var defaultWeightingCombination:=root.SelectSingleNode('/Simulation/GlobalParameters/WeightingCombination').InnerText;
  var defaultMonteCarloRuns:=Convert.ToInt32(root.SelectSingleNode('/Simulation/GlobalParameters/MonteCarloRuns').InnerText);
  _outputVerbal:=Convert.ToBoolean(root.SelectSingleNode('/Simulation/GlobalParameters/OutputVerbal').InnerText);
  _outputInputParameters:=Convert.ToBoolean(root.SelectSingleNode('/Simulation/GlobalParameters/OutputInputParameters').InnerText);
  
  var scenarioNodes:=root.SelectNodes('/Simulation/Scenarios/Scenario');
  var lScenarios:=new Dictionary<string,Scenario>;  
  
  var currentScenarioId:=0;
  for each scenarioNode: XmlNode in scenarioNodes do
  begin
    var scenario: Scenario:=new Scenario;
    scenario.Name:=(scenarioNode as XmlElement).GetAttribute('name');
    scenario.Id:=currentScenarioId;
    
    var excelfileNodes:=scenarioNode.SelectNodes('ExcelFile');    
    for each excelfileNode: XmlNode in excelfileNodes do
    begin
      var excelfilename: string := iif(Path.IsPathRooted(excelFileNode.InnerText), excelFileNode.InnerText, Path.Combine(Path.GetDirectoryName(Path.GetFullPath(_fileName)), excelfileNode.InnerText));
      scenario.ExcelFiles.Add(excelfilename);
    end;  
    
    lScenarios.Add(scenario.Name, scenario);
    _scenarios.Add(scenario);
   
    Inc(currentScenarioId);  
  end;
  
  var runNodes:=root.SelectNodes('/Simulation/Runs/Run');
  
  for each runNode: XmlNode in runNodes do
  begin
    var run:=new Run();
    
    run.Scenario:=lScenarios.Item[runNode.SelectSingleNode('Scenario').InnerText];
    
    if runNode.SelectSingleNode('MonteCarloRuns')<>nil then
      run.MonteCarloRuns:=Convert.ToInt32(runNode.SelectSingleNode('MonteCarloRuns').InnerText)
    else
      run.MonteCarloRuns:=defaultMonteCarloRuns;
    
    case runNode.SelectSingleNode('Mode').InnerText of
      'Marginal': run.Mode:=RunMode.MarginalRun;
      'Total': run.Mode:=RunMode.TotalRun;
      'CEA': run.Mode:=RunMode.CEARun;
      'FullMarginal': run.Mode:=RunMode.FullMarginalRun;
      'BGE': run.Mode:=RunMode.BGERun;
      'EpaMarginal': run.Mode:=RunMode.EpaMarginalRun;
      'Marginal2': run.Mode:=RunMode.MarginalRun2;
      'OptimalTaxation': run.Mode:=RunMode.OptimalTaxation;
      else raise new ApplicationException('Invalid value for Mode in configuration file');
    end;
    
    if runNode.SelectSingleNode('Scengmt')<>nil then
      run.Scengmt:=Convert.ToBoolean(runNode.SelectSingleNode('Scengmt').InnerText)
    else
      run.Scengmt:=defaultScenGmt;
      
   if runNode.SelectSingleNode('YearsToAggregate')<>nil then
     run.YearsToAggregate:=Convert.ToInt32(runNode.SelectSingleNode('YearsToAggregate').InnerText)
   else
     run.YearsToAggregate:=defaultYearsToAggregate;
     
   if runNode.SelectSingleNode('SwfToOptimise')<>nil then
     case runNode.SelectSingleNode('SwfToOptimise').InnerText of
       'Global': run.SWFToOptimise:=SWFtoOptimize.GlobalSwf;
       'Regional': run.SWFToOptimise:=SWFtoOptimize.RegionalSwf;
       else raise new ApplicationException('Invalid value for SwfToOptimise in configuration file');
     end
   else
     run.SWFToOptimise:=nil;
 
   if runNode.SelectSingleNode('CalculateMeanForMonteCarlo')<>nil then
     run.CalculateMeanForMonteCarlo:=Convert.ToBoolean(runNode.SelectSingleNode('CalculateMeanForMonteCarlo').InnerText)
   else
     run.CalculateMeanForMonteCarlo:=false;

   if runNode.SelectSingleNode('OutputAllMonteCarloRuns')<>nil then
     run.OutputAllMonteCarloRuns:=Convert.ToBoolean(runNode.SelectSingleNode('OutputAllMonteCarloRuns').InnerText)
   else
     run.OutputAllMonteCarloRuns:=true;                    
     
    if runNode.SelectSingleNode('WeightingCombination')<>nil then
      run.WeightingCombination:=runNode.SelectSingleNode('WeightingCombination').InnerText
    else
      run.WeightingCombination:=defaultWeightingCombination;       
   
   run.OutputVerbal:=_outputVerbal;
    
    _runs.Add(run);
  end;
end;

end.