// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

(* This unit contains a number of procedures to generate random variables *)

interface

type
  
  LegacyFund28Math = public static class
  private
    method GamLTOne(alpha: double; VAR a: double; rand: Random);    
    method GamGTOne(alpha: double; VAR a: double; rand: Random);    
  public
    method GenNormal(mu, sigma: double; VAR a,b: double; rand: Random);

    method GenNormGTZ(mu, sigma: double; VAR a,b: double; rand: Random);

    method GenNormKnot(mu, sigma, l, u: double; VAR a,b: double; rand: Random);

    method GenLogNormal(mu, sigma: double; VAR a,b: double; rand: Random);

    method GenGumbel(dzeta, theta: double; VAR a: double; rand: Random);

    method GenExponential(lambda: double; VAR a: double; rand: Random);

    method GenGamma(alpha,beta: double; VAR a: double; rand: Random);

    method GenPareto(alpha, kappa: double; VAR a: double; rand: Random);

    method GenTriang(a,b,c: double; VAR x: double; rand: Random);

    method Expo(x: double): double;

    method Pow(x,y: double): double;

    method Log(x: double): double;

    method ln(x: Double): Double;

    method sqrt(x: Double): Double;

    method Exp(x: Double): Double;

    method Sqr(x: Double): Double;
    
  end;

IMPLEMENTATION

method LegacyFund28Math.Sqr(x: Double): Double;
begin
  Result:=Math.Pow(x,2);
end;

method LegacyFund28Math.Exp(x: Double): Double;
begin
  Result:=Math.Exp(x);
end;

method LegacyFund28Math.sqrt(x: Double): Double;
begin
    Result:=Math.Sqrt(x);
end;

method LegacyFund28Math.ln(x: Double): Double;
begin
    Result:=Math.Log(x);
end;

method LegacyFund28Math.Log(x: double): double;
(* returns ln(x) *)

BEGIN
     IF (x > 1.0E-38) AND (x < 1.0E38) THEN
        Result:= ln(x)
     ELSE IF x < 1.0E-38 THEN
          Result:= ln(1.0E-38)
     ELSE
         Result:= ln(1.0E38)
END;

method LegacyFund28Math.Expo(x: double): double;
(* returns exp(x) *)

BEGIN
     IF (x < 88) AND (x > -88) THEN
        Result:= exp(x)
     ELSE IF x < -88 THEN
          Result:= exp(-88)
     ELSE
         Result:= exp(88)
END;

method LegacyFund28Math.Pow(x,y: double): double;
(* returns x^y *)

BEGIN
     Result:= expo(y*log(x))
END;

method LegacyFund28Math.GenNormal(mu, sigma: double; VAR a,b: double; rand: Random);

VAR u,v,w: double;

BEGIN
     u:= 2*rand.NextDouble()-1;
     v:= 2*rand.NextDouble()-1;
     w:= sqr(u)+sqr(v);
     IF w>1 THEN
        GenNormal(mu,sigma,var a,var b, rand)
     ELSE BEGIN
          w:= sqrt(-2*log(w)/w);
          a:= mu + sigma*w*u;
          b:= mu + sigma*w*v
     END
END;

method LegacyFund28Math.GenNormGTZ(mu, sigma: double; VAR a,b: double; rand: Random);

BEGIN
     GenNormal(mu,sigma,var a,var b, rand);
     WHILE (a < 0) OR (b < 0) DO
           GenNormal(mu,sigma,var a,var b, rand)
END;

method LegacyFund28Math.GenNormKnot(mu, sigma, l, u: double; VAR a,b: double; rand: Random);

BEGIN
     GenNormal(mu,sigma,var a,var b, rand);
     WHILE (a < l) OR (b < l) OR (a > u) OR (b > u) DO
           GenNormal(mu,sigma,var a,var b, rand)
END;

method LegacyFund28Math.GenLogNormal(mu, sigma: double; VAR a,b: double; rand: Random);

BEGIN
     GenNormal(mu,sigma,var a,var b, rand);
     a:= expo(a);
     b:= expo(b)
END;

method LegacyFund28Math.GenGumbel(dzeta, theta: double; VAR a: double; rand: Random);

BEGIN
     a:= dzeta - theta*log(-log(rand.NextDouble()))
END;

method LegacyFund28Math.GenExponential(lambda: double; VAR a: double; rand: Random);

BEGIN
     a:= -lambda*log(rand.NextDouble())
END;

method LegacyFund28Math.GamLTOne(alpha: double; VAR a: double; rand: Random);

VAR e,b,p,y: double;

BEGIN
     e:= expo(1);
     b:= (e + alpha)/e;
     P:= b*rand.NextDouble();
     IF P < 1 THEN BEGIN
        Y:= expo(log(P)/alpha);
        IF expo(-Y) >= rand.NextDouble() THEN
           a:= Y
        ELSE
            GamLTOne(alpha,var a, rand)
     END
     ELSE BEGIN
         Y:= -log((b-P)/alpha);
         IF expo((alpha-1)*log(Y)) >= rand.NextDouble() THEN
            a:= Y
         ELSE
             GamLTOne(alpha,var a, rand)
     END
END;

method LegacyFund28Math.GamGTOne(alpha: double; VAR a: double; rand: Random);

VAR ha, b, q, th, d, u1, u2, v, y, z, w: double;

BEGIN
     ha:= 1/sqrt(2*alpha-1);
     b:= alpha - log(4);
     q:= alpha + 1/ha;
     th:= 4.5;
     d:= 1 + log(th);
     u1:= rand.NextDouble();
     u2:= rand.NextDouble();
     v:= ha*log(u1/(1-u1));
     y:= alpha*expo(v);
     z:= u1*u1*u2;
     w:= b + q*v - y;
     IF (w + d - th*z) >= 0 THEN
        a:= y
     ELSE IF w >= log(z) THEN
          a:= y
     ELSE
         GamGTOne(alpha,var a, rand)
END;

method LegacyFund28Math.GenGamma(alpha,beta: double; VAR a: double; rand: Random);

BEGIN
     IF alpha < 1 THEN BEGIN
        GamLTOne(alpha,var a, rand);
        a:= beta*a
     END
     ELSE IF alpha = 1 THEN BEGIN
          GenExponential(1,var a, rand);
          a:= beta*a
     END
     ELSE BEGIN
         GamGTOne(alpha,var a, rand);
         a:= beta*a
     END
END;

method LegacyFund28Math.GenPareto(alpha, kappa: double; VAR a: double; rand: Random);

BEGIN
     a:= kappa*expo(-log(1-rand.NextDouble())/alpha)
END;

method LegacyFund28Math.GenTriang(a,b,c: double; VAR x: double; rand: Random);

VAR d,u,h: double;

BEGIN
     d:= (c-a)/(b-a);
     u:= rand.NextDouble();
     IF u <= d THEN
        h:= sqrt(d*u)
     ELSE
         h:= 1-sqrt((1-d)*(1-u));
     x:= a+(b-a)*h
END;

END.